"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_rest_1 = require("typescript-rest");
const contextConstructorName = "ServiceContext";
const authConstructorName = "Auth";
const decodedConstructorName = "DecodedToken";
/**
 * Add Firebase authentication to a route
 *
 * @param target The prototype of the class
 * @param propertyKey The name of the method
 * @param descriptor The descriptor
 */
function FireAuth() {
    return function (target, propertyKey, descriptor) {
        //save original method.
        const originalMethod = descriptor.value;
        let serviceContext;
        let auth;
        let tokenPropName;
        descriptor.value = function () {
            return new Promise((resolve, reject) => {
                //get typescript-rest servicecontext,firebase auth, and decoded token from controller object.
                for (let propName in this) {
                    if (this[propName].constructor.name === contextConstructorName) {
                        serviceContext = this[propName];
                    }
                    if (this[propName].constructor.name === authConstructorName) {
                        auth = this[propName];
                    }
                    if (this[propName].constructor.name === decodedConstructorName) {
                        tokenPropName = propName;
                    }
                }
                if (serviceContext && auth) {
                    let bearerToken = serviceContext.request.get('Authorization');
                    if (bearerToken
                        && ~bearerToken.indexOf('Bearer')
                        && bearerToken.split(' ').length === 2
                        && bearerToken.split(' ')[1] !== '') {
                        auth.verifyIdToken(bearerToken.split(' ')[1])
                            .then((decodedToken) => {
                            //token is valid. 
                            //assign value of decoded token to controller prop
                            if (tokenPropName) {
                                Object.assign(this[tokenPropName], decodedToken);
                            }
                            //run original method.
                            var result = originalMethod.apply(this, arguments);
                            resolve(result);
                        })
                            .catch((err) => {
                            //token is invalid, reject.
                            console.error(err.message);
                            reject(new typescript_rest_1.Errors.UnauthorizedError('Invalid token.'));
                        });
                    }
                    else {
                        //no bearer token sent, reject.`
                        reject(new typescript_rest_1.Errors.UnauthorizedError('Bearer authentication is required for this endpoint.'));
                    }
                }
                else {
                    console.error('typescript-rest-fireauth error: Required controller properties not found. Did you add ServiceContext and admin.auth.Auth properties to your controller? See README.');
                    reject(new typescript_rest_1.Errors.InternalServerError());
                }
            });
        };
        return descriptor;
    };
}
exports.FireAuth = FireAuth;
class DecodedToken {
    constructor() { }
    ;
}
exports.DecodedToken = DecodedToken;
